@@ -1,9 +1,32 @@
-# $Id: Expressions.pm 1046 2008-06-04 19:28:02Z tinita $
+package HTML::Template::Compiled::Expression::Expressions;
+# $Id: Expressions.pm,v 1.4 2006/06/20 21:12:24 tinita Exp $
+use strict;
+use warnings;
+use base 'Exporter';
+our @EXPORT = qw(
+    &_expr_close
+    &_expr_literal
+    &_expr_defined
+    &_expr_ternary
+    &_expr_string
+    &_expr_function
+    &_expr_method
+);
+use HTML::Template::Compiled::Expression::Conditional;
+use HTML::Template::Compiled::Expression::Function;
+
+sub _expr_close { HTML::Template::Compiled::Expression::Close->new }
+sub _expr_literal { HTML::Template::Compiled::Expression::Literal->new(@_) }
+sub _expr_defined { HTML::Template::Compiled::Expression::Defined->new(@_) }
+sub _expr_ternary { HTML::Template::Compiled::Expression::Ternary->new(@_) }
+sub _expr_string { HTML::Template::Compiled::Expression::String->new(@_) }
+sub _expr_function { HTML::Template::Compiled::Expression::Function->new(@_) }
+sub _expr_method { HTML::Template::Compiled::Expression::Method->new(@_) }
+
 
 package HTML::Template::Compiled::Expression::Close;
 use strict;
 use warnings;
-our $VERSION = '0.01';
 use base qw(HTML::Template::Compiled::Expression);
 
 sub to_string {
@@ -12,17 +35,6 @@ sub to_string {
     return $indent . '}';
 }
 
-package HTML::Template::Compiled::Expression::Open;
-use strict;
-use warnings;
-use base qw(HTML::Template::Compiled::Expression);
-
-sub to_string {
-    my ($self, $level) = @_;
-    my $indent = $self->level2indent($level);
-    return $indent . '{';
-}
-
 
 package HTML::Template::Compiled::Expression::Defined;
 use strict;
@@ -70,7 +82,6 @@ sub to_string {
     my ($op) = $self->get_operands;
     my $dump = Data::Dumper->Dump([\$op], ['op']);
     $dump =~ s#^\$op *= *\\##;
-    $dump =~ s/;$//;
     return $dump;
 }
 
